'''
Copyright (C) 2015 Rene Grzeszick. All rights reserved.
This file made available under the terms of the BSD license (see the Copyright.txt).

Created on Jun 30, 2014

********************************************************************
This script contains a demo for the encoding of time coordinates using 
synthesized keypoints at different locations.

@author: rgrzeszi
'''
import numpy as np

import matplotlib.pyplot as plt
import matplotlib.animation as animation

from bof.spatial_features import SpatialFeatureCreator



if __name__ == '__main__':

    NUM_DENSITIES_GEN = 2
    NUM_SAMPLES = 1000

    # Initialize config object
    # Set pseudo names for generated feat. representations
    # Set number of clusters (10 per repr.)
    localizations = np.array([[0.001,0.001],
                              [0.1, 0.1], [0.1, 0.25], [0.1, 0.5], [0.1, 0.75], [0.1, 0.9],
                              [0.25, 0.1], [0.25, 0.25], [0.25, 0.5], [0.25, 0.75], [0.25, 0.9],
                              [0.5, 0.1], [0.5, 0.25], [0.5, 0.5], [0.5, 0.75], [0.5, 0.9],
                              [0.75, 0.1], [0.75, 0.25], [0.75, 0.5], [0.75, 0.75], [0.75, 0.9],
                              [0.9, 0.1], [0.9, 0.25], [0.9, 0.5], [0.9, 0.75], [0.9, 0.9],
                              [0.999,0.999]])
    pseudo_descr = []
    for _ in localizations:
        pseudo_descr.append([1,1])
    pseudo_descr = np.array(pseudo_descr)
    
    fig, axis_arr = plt.subplots(1, 2, sharey=True)
    axis_arr[0].set_title("(x,y)")
    axis_arr[0].scatter(localizations[:,0], localizations[:,1], marker='x', color='b')
    axis_arr[1].set_title("(r, sin(alpha), cos(alpha))")
    axis_arr[1].scatter(localizations[:,0], localizations[:,1], marker='x', color='b')
    
    

    spatial_trafo = SpatialFeatureCreator("xy", 2, ".meta")
    len_spatial = spatial_trafo.get_len_spatial_component()
    descriptors_xy = spatial_trafo.add_spatial_feature(localizations, pseudo_descr, randomize=False)
    descriptors_xy = np.array(descriptors_xy, 'f')
    
    #print("XY coordinates")
    for l, d_i in zip(localizations, descriptors_xy):
        axis_arr[0].text(l[0], l[1], str(d_i[-len_spatial:])[1:-2],
        horizontalalignment='left',
        verticalalignment='center')
        #print l, d_i[-len_spatial:]
    axis_arr[0].set_xlim(-0.1,1.1)
    axis_arr[0].set_ylim(-0.1,1.1)
    
    spatial_trafo = SpatialFeatureCreator("radial", 2, ".meta")
    len_spatial = spatial_trafo.get_len_spatial_component()
    descriptors_rad = spatial_trafo.add_spatial_feature(localizations, pseudo_descr, randomize=False)
    descriptors_rad = np.array(descriptors_rad, 'f')
        
    #print("Radial coordinates")
    for l, d_i in zip(localizations, descriptors_rad):
        axis_arr[1].text(l[0], l[1], " "+str(d_i[-len_spatial:])[1:-1],
        horizontalalignment='left', verticalalignment='center',
        fontsize='smaller')
        #print l, d_i[-len_spatial:]
    axis_arr[1].set_xlim(-0.1,1.1)
    axis_arr[1].set_ylim(-0.1,1.1)
    
    plt.show()
