'''
Created on Nov 7, 2012

Dense SIFT extraction Demo

@author: rgrzeszi
'''

import Image
import numpy, time
import vlfeat
from numpy.ma.core import floor, zeros, sum
from numpy.core.numeric import concatenate

tstart = time.time()

I = Image.open("./sample.png")
I = vlfeat.vl_rgb2gray(numpy.array(I))    
I = numpy.array(I, 'f', order='F') # 'F' = column-major order!


verbose = True
fast = True
sizes = [4, 6, 8, 10]
step = 2
color = 'gray' 
numChannels = 1
floatdescriptors = False
magnif = 6 
windowsize = 1.5 
contrastthreshold = 0.005    

#dse = DenseSiftExtractor(sizes, step, magnif, fast, windowsize, contrastthreshold, verbose)
#[l, d] = dse.getDenseSIFT('/home/rgrzeszi/sources/vlfeat-python-wrappers/data/a.jpg')

#Output
td = []

#I = loadtxt("/home/rgrzeszi/imdata", delimiter=",")
#I = numpy.array(I, 'f', order='F')

for si in sizes:

    '''
    % Recall from VL_DSIFT() that the first descriptor for scale SIZE has
    % center located at XC = XMIN + 3/2 SIZE (the Y coordinate is
    % similar). It is convenient to align the descriptors at different
    % scales so that they have the same geometric centers. For the
    % maximum size we pick XMIN = 1 and we get centers starting from
    % XC = 1 + 3/2 MAX(OPTS.SIZES). For any other scale we pick XMIN so
    % that XMIN + 3/2 SIZE = 1 + 3/2 MAX(OPTS.SIZES).
    %
    % In pracrice, the offset must be integer ('bounds'), so the
    % alignment works properly only if all OPTS.SZES are even or odd.
    '''
    
    off = floor(1 + 3.0/2.0 * (max(sizes) - si)) 
    

    '''
    % smooth the image to the appropriate scale based on the size
    % of the SIFT bins
    '''
    
    sigma = float(si) / magnif 
    ims = vlfeat.vl_imsmooth(I, sigma)
    
    #Create proper bounds
    bounds = zeros(4,'f')
    bounds[0] = off
    bounds[1] = off
    bounds[2] = len(ims)
    bounds[3] = len(ims[0])
    
    #% extract dense SIFT features from all channels
    #I = [I]
    for k in range(numChannels):
        #        vlfeat.vl_dsift(data, step, bounds, size, fast, verbose, norm)
        [f, d] = vlfeat.vl_dsift(I, step=step, bounds=bounds, size=int(si), fast=fast, windowsize=windowsize, verbose=verbose, norm=True)
         

    contrast = f[2,:];
    d[:, contrast < contrastthreshold] = 0 ;
    
    if si == sizes[0]:
        print(d[:,0])
    
    if td == []:
        td = d
    else:
        td = concatenate((td,d),axis=1)

tend = time.time()

print(len(td[0]))
print(sum(sum(td)))
print("runtime: %.2f" % (tend - tstart))